# Julia Yu, Game Engine

import time, random
from decimal import *

class State:
    def __init__(self, name, description):
        self.name = name
        self.description = description
        self.edges = []
        self.stuffs = []

class Edge:
    def __init__(self, name, from_node, commands, to_node, health, desc, constrNames=None):
        self.name=name
        self.from_node = from_node
        self.commands = commands
        self.to_node = to_node
        self.health=health
        self.desc=desc
        self.constrNames = constrNames
        self.constrs={}

class Object:
    def __init__(self, homeNode, name, shortDesc, desc, weight):
        self.homeNode=homeNode
        self.name=name
        self.shortDesc=shortDesc
        self.desc=desc
        self.weight=weight

class Constraint:
    def __init__(self, name, mode, condition):
        self.name=name
        self.mode=mode
        self.condition=condition

class Pack:
    def __init__(self, maxWeight, value):
        self.maxWeight=maxWeight
        self.value=value
        self.stuffs=[]
    def names(self):
        n=[]
        for item in self.stuffs:
            n.append(item.name)
        return n

class Engine:
    def __init__(self, filename):
        self.filename = filename
        self.initial_time=time.time()
        self.initial_state = None
        self.pack=None

    def load(self):
        self.states = {}
        constraints=[]
        edges = []
        stuffs= []
        fp = open(self.filename)
        for line in fp:
            if len(line) == 0 or line[0] == '#':
                continue
            items = line.strip().split("::")
            if items[0] == 'node':
                s = State(*items[1:])
                if self.initial_state is None:
                    self.initial_state = s
                self.states[items[1]] = s
            elif items[0]=='backpack':
                pack=Pack(*items[1:])
            elif items[0]=='constraint':
                constraints.append(Constraint(*items[1:]))
            elif items[0] == 'edge':
                edges.append(Edge(*items[1:]))
            elif items[0]=='item':
                stuffs.append(Object(*items[1:]))
        for edge in edges:
            self.states[edge.from_node].edges.append(edge)
            if edge.constrNames!=None:
                eSplit=edge.constrNames.split(",")
                for name in eSplit:
                    for constr in constraints:
                        if constr.name==name:
                            edge.constrs[constr.name]=constr
        for obj in stuffs:
            self.states[obj.homeNode].stuffs.append(obj)

    def play(self):
        current_state = self.initial_state
        #Creates a random pack if none is assigned.
        if self.pack==None:
            self.pack=Pack(10, 5)
        pack=self.pack
        count=0
        cWord="Command: "
        end=False

        #HEALTH
        health=0
        healthYN=raw_input("Does your game use health (Yes/No)?: ")
        if healthYN=='Yes' or healthYN=='yes' or healthYN=='y':
            health=int(raw_input("What is your starting health? "))
        else:
            health=None

        #WRAP AROUND TEXT
        def noBreak(text, col=75):
            wrapText=""
            count=0
            for letter in text:
                if count>col and letter==" ":
                    wrapText=wrapText+"\n"
                    count=0
                else:
                    wrapText=wrapText+letter
                    count=count+1
            return wrapText

        def endCredits():
            print(" ")
            print(" ")
            print(" ")
            print("Engine and Text File by J.Yoo")
            print("'You and Yours' plot by Ellen MacInnis, J.Yoo")
            print("Thanks to: Shayna Schwartz, Emma DiQuinzio, and Nancy Yu")
    
        while end==False:
            trufalse=0
            repeat=True
            print(noBreak(current_state.description))
            command = raw_input(cWord).lower()
            csplit=command.split()

            #INVENTORY
            if command=="memory" or command=="inventory":
                print("You remember...")
                if len(pack.stuffs)!=0:
                    for thing in pack.stuffs:
                        print(thing.shortDesc)
                else:
                    print("...nothing.")
                trufalse=1

            #PLAYER CHECKS
            if len(csplit)==0:
                print("...what?")
            if len(csplit)==1 and csplit[0]=='quit':
                if raw_input("Are you sure you want to quit? Yes/No ") =="yes".lower():
                    print("Hurry back.")
                    return
            if len(csplit)==1 and csplit[0]=='look':
                print("Look at what?")
                repeat=False
            if len(csplit)==1 and csplit[0]=='ask':
                print("What do you want to ask?")
                repeat=False
            if (len(csplit)==3 and csplit[:3]=='look at folder') or (len(csplit)==2 and csplit[:2]=="look folder"):
                print("Look at which folder?")
            if len(csplit)==1 and csplit[0]=='time':
                print("You have been playing for " + str(Decimal(str(time.time()-self.initial_time)).quantize(Decimal('.01'), rounding=ROUND_UP)
) + " seconds")
                repeat=False
            if len(csplit)==1 and csplit[0] =='count':
                if count==1:
                    print("You have made " + str(count) + " move.")
                else:
                    print("You have made " + str(count) + " moves.")
                repeat=False
            if health==0:
                print("You have run out of health.  You die.")
                return
                

            #EDGES
            for edge in current_state.edges:
                if repeat==True:
            #CONSTRAINTS
                    if len(edge.constrs.keys())>0:
                        truList=[]
                        numExe=0
                        for constr in edge.constrs.keys():
            #EVAL CONSTRAINTS
                            if edge.constrs[constr].mode=='eval':
                                if eval(edge.constrs[constr].condition):
                                    truList.append(True)
            #EXEC CONSTRAINTS
                            elif edge.constrs[constr].mode=='exec':
                                numExe=numExe+1
                            else:
                                if edge.constrs[constr].condition!='backpack':
                                    for item in self.states[edge.constrs[constr].condition].stuffs:
                                        if edge.constrs[constr].mode==item.name:
                                            truList.append(True)
                                else:
                                    if edge.constrs[constr].mode in pack.names():
                                        truList.append(True)
            #If the constraints are met...
                        if (command in edge.commands):
                            if len(truList)==(len(edge.constrs.keys())-numExe):
                                if edge.constrs[constr].mode=='exec':
                                    exec(edge.constrs[constr].condition)
                                else:
                                    current_state = self.states[edge.to_node]
                                    print(noBreak(edge.desc))
                                    count=count+1
            #HEALTH
                                    if health!=None:
                                        health=health+edge.health
                            else:
                                print("You can't do that just yet.")
                            trufalse=1
            #If there are no constraints...
                    elif command in edge.commands:
                            current_state = self.states[edge.to_node]
                            print(noBreak(edge.desc))
                            trufalse=1
                            count=count+1
            #HEALTH
                            if health!=None:
                                health=health+int(edge.health)
                                
            #BACKPACK
            if pack!=None:
                for stuff in pack.stuffs:
                    if (csplit[0]=='pickup' or csplit[0]=='memorize') and csplit[1:]==stuff.name.split():
                        print("You already committed this piece of information to memory.")
                        trufalse=1
                    elif (csplit[0]=='drop' or csplit[0]=='forget') and csplit[1:]==stuff.name.split():
                        print("You let the " + stuff.shortDesc +" go.")
                        pack.stuffs.remove(stuff)
                        current_state.stuffs.append(stuff)
                        trufalse=1
                for stuff in current_state.stuffs:
                    if (csplit[0]=='pickup' or csplit[0]=='memorize') and csplit[1:]==stuff.name.split():
                        print(stuff.desc)
                        current_state.stuffs.remove(stuff)
                        pack.stuffs.append(stuff)
                        trufalse=1

            #EDGE CHECK: does what you typed make sense?
            if trufalse==0:
                if repeat==True:
                    print("You're confused, you're not making sense.  Maybe you should try something else...")    

        if end==True:
            endCredits()
    
game = Engine("gameText.txt")
game.load()
game.play()
